<?php
/**
 * @file
 * Regular expression based implementation of the OAI-PMH parser.
 * 
 * The XML response is parsed with regular expressions.
 * 
 * @author Király Péter <kirunews@gmail.com>
 * 
 * @copyright (c) 2010-2011 eXtensible Catalog Organization
 */

class OAIRegexParser  implements IOAIParser {

  private $raw_content;
  private $raw_records;
  private $raw_resumption;
  private $http_code;

  private $raw_errors       = "";
  private $errorMsgs        = array();
  private $statistics       = array();
  private $has_errors       = FALSE;
  private $has_list_records = FALSE;
  private $has_records      = FALSE;
  private $record_count     = 0;
  private $records          = NULL;
  private $record_counter   = -1;
  private $resumptionToken;
  private $has_resumptionToken = FALSE;


  /* PUBLIC FUNCTIONS */
  function set_http_code($http_code) {
    $this->http_code = $http_code;
  }

  function get_http_code() {
    return $this->http_code;
  }
    
  function process($content, $http_code = 200) {
    $this->http_code = $http_code;
    $this->raw_content = $content;
    if ($http_code == 200) {
      $this->raw_content = preg_replace('/<!--.*?-->/', '', $this->raw_content);
      $this->_process_content();
    }
    else {
      $this->errorMsg[] = $this->raw_content;
    }
  }

  function processFile($fileName) {
    $this->statistics['t0'] = microtime(TRUE);
    $this->raw_content = file_get_contents($fileName);
    $this->statistics['t1'] = microtime(TRUE);
    $this->_process_content();
  }

  function has_errors() {
    return $this->has_errors  || ((int)$this->http_code != 200);
  }
  
  function get_internal_errors() {
    return $this->errorMsgs;
  }

  
  function list_errors() {
    $error = new OaiPmh_Dom_Error($this->raw_content);
    $oai = $error->getOaiPmh();
    return $oai['error'];
  }

  function has_errorMsg() {
    return empty($this->errorMsg);
  }

  function get_errorMsg() {
    return $this->errorMsg;
  }
  
  function has_records() {
    return $this->has_records;
  }

  function get_record_count() {
    return $this->record_count;
  }

  function get_next_record() {
    if ($this->record_counter == -1) {
      $this->statistics['foreach'] = array(
        'new' => 0.0,
        'parse' => 0.0,
        'dom_set_schemas' => 0.0,
        'dom_set_dom' => 0.0,
      );
    }
    if (++$this->record_counter < $this->record_count) {
      //$xml = '<record>' . $this->records[$this->record_counter];
      $xml = $this->records[$this->record_counter];
      //if (!$domDocument = @DOMDocument::loadXML($xml)) {
      //  throw new Exception('Unknown DOMDocument::loadXML() error.: ' . $this->records[$this->record_counter]);
      //}
      $t0 = microtime(TRUE);
      $record = new OaiPmh_Dom_ListRecords($xml, TRUE);
      $t1 = microtime(TRUE);
      $recordArray = $record->parseRecord(NULL, FALSE);
      $t2 = microtime(TRUE);
      $this->statistics['foreach']['new'] += ($t1 - $t0);
      $this->statistics['foreach']['parse'] += ($t2 - $t1);
      $rstat = $record->getStatistics();
      $this->statistics['foreach']['dom_set_schemas'] += $rstat['dom_set_schemas'];
      $this->statistics['foreach']['dom_set_dom'] += $rstat['dom_set_dom'];
      /*
      if ($this->record_counter == 1) {
        print_r($recordArray);
      }
      */
      return $recordArray;
    }
    else {
      return FALSE;
    }
  }

  function has_resumptionToken() {
    return $this->has_resumptionToken;
  }

  function get_resumptionToken() {
    return $this->resumptionToken;
  }

  function getStatistics() {
    return $this->statistics;
  }
  
  function print_statistics() {
    $t = $this->statistics;
    $msg = 'total: ' . sprintf("%.3f", ($t['tend'] - $t['t0'])) . "\n";
    $msg .= '1: ' . sprintf("%.3f", ($t['t1'] - $t['t0'])) . "\n";
    $msg .= '2: ' . sprintf("%.3f", ($t['tend'] - $t['t1'])) . "\n";
    if (isset($t['trec2'])) {
      $msg .= '3: ' . sprintf("%.3f", ($t['trec2'] - $t['trec1'])) . "\n";
    }
    return $msg;
  }

  
  /* PRIVATE FUNCTIONS */
  private function _process_content() {
    if (strstr($this->raw_content, '<error') && !strstr($this->raw_content, '<ListRecords>')) {
      $text = $this->raw_content;
    }
    else {
      $text = $this->raw_content;
      $this->raw_content = '';
    }
    $size = drupal_strlen($text);
    $text = preg_replace("/^<\?xml[^<>]+\?>/", "", $text);
    if ($size == drupal_strlen($text)) {
      xc_log_error('regex', 'no sheabang');
      $this->errorMsgs[] = 'no sheabang';
    }

    $size = drupal_strlen($text);
    $text = preg_replace("/^\s*<OAI-PMH\b[^<>]+>/s", "", $text);
    if ($size == drupal_strlen($text)) {
      xc_log_error('regex', 'no &lt;OAI-PMH&gt;');
      $this->errorMsgs[] = 'no OAI-PMH';
      return;
    }

    $size = drupal_strlen($text);
    $text = preg_replace("/^\s*<responseDate>(.*?)<\/responseDate>/s", "", $text);
    if ($size == drupal_strlen($text)) {
      xc_log_error('regex', 'no &lt;responseDate&gt;');
      $this->errorMsgs[] = 'no responseDate';
      return;
    }

    $size = drupal_strlen($text);
    $text = preg_replace("/^\s*<request(?: [^<>]+)?>(.*?)<\/request>/s", "", $text);
    if ($size == drupal_strlen($text)) {
      xc_log_error('regex', 'no &lt;request&gt; element in XML response: ' 
        . htmlentities(substr($text, 0, 50)) . '&hellip;');
      $this->errorMsgs[] = 'no request';
      return;
    }

    $this->has_list_records = FALSE;
    $this->has_errors = FALSE;
    $size = drupal_strlen($text);
    $text = preg_replace("/^\s*<ListRecords>/", "", $text);
    if ($size == drupal_strlen($text)) {
      $errors = '';
      $size = drupal_strlen($text);
      $text = preg_replace_callback("/^\s*<error(?: code=([\'\"])(.*)([\'\"]))?>(.*)<\/error>/", 
        array($this, 'set_errors'), $text);
      if ($size == drupal_strlen($text)) {
        xc_log_error('regex', 'no ListRecords and error');
        $this->errorMsgs[] = 'no ListRecords and error';
      }
      else {
        $this->has_errors = TRUE;
      }
    }
    else {
      $this->has_list_records = TRUE;
    }
    if (!$this->has_errors) {
      $this->raw_content = '';
    }

    if ($this->has_list_records) {
      $this->statistics['trec1'] = microtime(TRUE);
      $resumption = '';
      $records    = '';
      $is_changed_size = TRUE;
      while (!preg_match("/^\s*<\/ListRecords>/", $text) && $is_changed_size) {
        $size = drupal_strlen($text);
        $text = preg_replace_callback("/^\s*(<record>.*<\/record>)/s", array($this, 'set_records'), $text);
        
        if ($size != drupal_strlen($text)) {
          $this->has_records = TRUE;
          $is_changed_size = TRUE;
          // raw_records is initialized by set_records
          $mod_records = str_replace('<record>', '<#######><record>', $this->raw_records);
          $this->records = preg_split("/<#######>/", $mod_records, -1, PREG_SPLIT_NO_EMPTY);
          $this->record_count = count($this->records);
          $mod_records = '';

          $size = drupal_strlen($text);
          $text = preg_replace_callback("/^\s*(<resumptionToken([^<>]*)>(.*?)<\/resumptionToken>)/", array($this, 'set_resumption'), $text);
          if ($size != drupal_strlen($text)) {
            $is_changed_size = TRUE;
            // resumption comes from set_resumption
            //echo 'recCount: ', $this->record_count, ", resumption: ", $this->resumptionToken['text'], "\n";
          }
          else {
            $is_changed_size = FALSE;
            //$this->errorMsgs[] = 'no resumptionToken';
          }
        }
        else {
          $is_changed_size = FALSE;
          $this->errorMsgs[] = 'no records';
        }
      } // process inside ListRecords

      $this->statistics['trec2'] = microtime(TRUE);

      $size = drupal_strlen($text);
      $text = preg_replace('/^\s*<\/ListRecords>/', '', $text);
      if ($size == drupal_strlen($text)) {
        $this->errorMsgs[] = 'no /ListRecords';
      }
    } // has ListRecords
    if ($this->has_list_records || $this->has_errors) {
      $size = drupal_strlen($text);
      $text = preg_replace("/^\s*<\/OAI-PMH>/", '', $text);
      if ($size == drupal_strlen($text)) {
        $this->errorMsgs[] = 'no /OAI-PMH';
      }
    } // has ListRecords or error
    $this->statistics['tend'] = microtime(TRUE);
    //$this->print_statistics();
  }

  /**
   * Save the error message come from OAI-PMH data provider
   * 
   * @param $matches (Array)
   *   matches[2]: the error code (optional)
   *   matches[4]: the error message
   * @return (string)
   *   An empty string to replace the error message
   */
  private function set_errors($matches) {
    $this->errorMsgs[] = t('OAI-PMH data provider sent an error message.')
     . ' '
     . (isset($matches[2]) ? $matches[2] . ': ' : '') . $matches[4];
    return "";
  }

  private function set_records($matches) {
    $this->raw_records = $matches[1];
    return "";
  }

  private function set_resumption($matches) {
    $xml = simplexml_load_string($matches[1]);
    $this->resumptionToken = array(
      'attributes' => array(
        'completeListSize' => (int)$xml['completeListSize'],
        'cursor'           => (int)$xml['cursor'],
        'expirationDate'   => (string)$xml['expirationDate'],
      ),
      'text'               => (string)$xml,
    );
    $this->has_resumptionToken = !empty($xml);
    return "";
  }
}
