<?php
/**
 * Copyright 2008 Center for History and New Media
 *
 * This file is part of OAI-PMH Parser Library.
 *
 * OAI-PMH Parser Library is free software: you can redistribute it
 * and/or modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation, either version 3 of
 * the License, or (at your option) any later version.

 * OAI-PMH Parser Library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with OAI-PMH Parser Library.  If not, see
 * <http://www.gnu.org/licenses/>.
 */

// See: http://us2.php.net/dom
/**
 * The basic class for the response objects
 */
abstract class OaiPmh_Dom extends OaiPmh {

  protected $_oaiPmhDomDocument;
  protected $_oaiPmh;
  protected $_schemas;
  protected $_isFragment = FALSE;
  protected $_statistics = array();

  /**
   * Creates a response object from the OAI-PMH response
   * @param $response
   * @return unknown_type
   */
  public function __construct($response, $isFragment = FALSE) {
    // Set the provided schemas
    $t0 = microtime(TRUE);
    if (!$isFragment) {
      $this->_setSchemas();
    }
    $this->_isFragment = $isFragment;

    $t1 = microtime(TRUE);
    $this->_statistics['dom_set_schemas'] = $t1 - $t0;
    // Set the DOM document
    $this->_setDomDocument($response);
    $t2 = microtime(TRUE);
    $this->_statistics['dom_set_dom'] = $t2 - $t1;
    
    if (!$isFragment) {
      // Set the OAI-PMH response date
      $this->_setResponseDate();
      $t3 = microtime(TRUE);
      $this->_statistics['dom_set_date'] = $t3 - $t2;
      
      // Set the OAI-PMH request
      $this->_setRequest();
      $t4 = microtime(TRUE);
      $this->_statistics['dom_set_request'] = $t4 - $t3;
    }
  }

  /**
   * Returns the array of values
   * @return unknown_type
   */
  public function getOaiPmh() {
    return $this->_oaiPmh;
  }

  /**
   * Check if the response is an error response. 
   * This is called from OaiPmh_Response, therefore a static method 
   * is necessary.
   * @param $response
   * @return unknown_type
   */
  public static function isErrorResponse($response) {

    // Suppress errors from DOMDocument::loadXML()
    if (!$domDocument = @DOMDocument::loadXML($response)) {
      throw new Exception($response);
    }

    if ($domDocument->getElementsByTagNameNS(parent::NAMESPACE_URI, 
        'error')->item(0) === NULL) {
      return false;
    }
    else {
      return true;
    }
  }

  /**
   * Set the DOM document
   * @param $response
   * @return unknown_type
   */
  protected function _setDomDocument($response) {
    // Suppress errors from DOMDocument::loadXML()
    if (!$this->_oaiPmhDomDocument = @DOMDocument::loadXML($response)) {
      throw new Exception('OAI-PMH parser error: Unknown error when '
        . 'loading XML into the DOM.');
    }
  }

  /**
   * Set the OAI-PMH response date.
   * @return unknown_type
   */
  protected function _setResponseDate() {
    $responseDateDomElement = $this->_oaiPmhDomDocument->getElementsByTagNameNS(
      parent::NAMESPACE_URI, 'responseDate')->item(0);
    $this->_oaiPmh['responseDate'] = $responseDateDomElement->nodeValue;
  }

  /**
   * Set the OAI-PMH request.
   * @return unknown_type
   */
  protected function _setRequest() {
    $requestDomElement = $this->_oaiPmhDomDocument->getElementsByTagNameNS(
      parent::NAMESPACE_URI, 'request')->item(0);
    if (!is_null($requestDomElement)) {
      $this->_oaiPmh['request'] = array(
        'attributes' => $this->_getDomElementAttributes($requestDomElement), 
        'text'       => $requestDomElement->nodeValue
      );
    }
    else {
      throw new Exception('OAI-PMH parser error: There is no' 
        . '\'' . parent::NAMESPACE_URI . '::request\' element in the OAI response.');
    }
  }

  /**
   * Get the OAI-PMH resumption token. Called from various children of 
   * this class.
   * @param $verbDomElement
   * @return unknown_type
   */
  protected function _getResumptionToken(DOMElement $verbDomElement) {
    $resumptionTokenDomElement = $verbDomElement->getElementsByTagName(
      'resumptionToken')->item(0);
    // The resumption token element must have a value in order for it to be
    // valid.
    if ($resumptionTokenDomElement instanceof DOMElement 
      && $resumptionTokenDomElement->nodeValue) {
      $resumptionToken = array(
        'attributes' => $this->_getDomElementAttributes(
          $resumptionTokenDomElement), 
        'text' => $resumptionTokenDomElement->nodeValue
      );
    }
    else {
      $resumptionToken = NULL;
    }
    return $resumptionToken;
  }

  /**
   * Get the attributes from a DOM element. Called from various children 
   * of this class.
   * @param $domElement
   * @return unknown_type
   */
  protected function _getDomElementAttributes(DOMElement $domElement) {
    $attributesDomNamedNodeMap = $domElement->attributes;
    $attributes = array();
    foreach ($attributesDomNamedNodeMap as $attribute) {
      $attributes[$attribute->name] = $attribute->value;
    }

    return $attributes;
  }

  /**
   * Returns the DOM element object of the first XML_ELEMENT_NODE it detects.
   * @param $domElement
   * @return unknown_type
   */
  protected function _getChildNode(DOMElement $domElement) {
    foreach ($domElement->childNodes as $childNode) {
      if ($childNode->nodeType == XML_ELEMENT_NODE) {
        return $childNode;
      }
    }
  }

  /**
   * Returns the namespace URI of the first XML_ELEMENT_NODE it detects.
   * @param $domElement
   * @return unknown_type
   */
  protected function _getChildNodeNamespaceUri(DOMElement $domElement) {
    foreach ($domElement->childNodes as $childNode) {
      if ($childNode->nodeType == XML_ELEMENT_NODE) {
        return $childNode->namespaceURI;
      }
    }
  }

  /**
   * Parse child nodes of <metadata>, <description>, and <about> elements 
   * based on their (required) namespace URI.
   * @param $domElement
   * @return unknown_type
   */
  protected function _getChildNodeByNamespaceUri(DOMElement $domElement, 
    $isFragment = FALSE) {
      
    // Set the child node
    $childNode = $this->_getChildNode($domElement);

    // Set the child node's namespace URI. Remove trailing forward 
    // slashes from the detected namespace.
    $childNodeNamespaceUri = preg_replace('/\/$/', '', 
      $this->_getChildNodeNamespaceUri($domElement));

    // Begin building the child node array.
    $childNodeArray = array('namespaceURI' => $childNodeNamespaceUri);

    // Iterate the schema array. If the namespaces match, call the 
    // particular Schema_* object and get the parsed XML.
    foreach ($this->_schemas as $schema) {
      // prevent parsing DC
      if ($schema['schemaName'] == 'OaiDc') {
        continue;
      }

      // Remove trailing forward slashes from the schema namespace.
      $schemaNamespaceUri = preg_replace('/\/$/', '', $schema['namespaceUri']);

      // If the schema URIs match, parse the XML using the appropriate schema.
      if ($schemaNamespaceUri == $childNodeNamespaceUri) {
        // Set the child node's XML
        $childNodeXml = $this->_oaiPmhDomDocument->saveXML($childNode);
        $childNodeArray['schemaName'] = $schema['schemaName'];
        $childNodeArray['childNode'] = Schema::factory($schema['schemaName'], 
          $childNodeXml);
        break;
      }

      // If none of the schema URIs match, assign the DOM child node to the
      // child node array.
      $childNodeArray['childNode'] = $childNode;
    }

    return $childNodeArray;
  }
  
  public function getStatistics() {
    return $this->_statistics;
  }
}
