<?php
/**
 * Copyright 2008 Center for History and New Media
 *
 * This file is part of OAI-PMH Parser Library.
 *
 * OAI-PMH Parser Library is free software: you can redistribute it 
 * and/or modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation, either version 3 of 
 * the License, or (at your option) any later version.

 * OAI-PMH Parser Library is distributed in the hope that it will be 
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty 
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with OAI-PMH Parser Library.  If not, see 
 * <http://www.gnu.org/licenses/>.
 */

class OaiPmh_Response extends OaiPmh
{
    protected $_requestVerb;
    protected $_requestUrl;
    protected $_response;
    protected $_http_code;
    private $_http_cache;
    private $_statistics = array();
    
    /**
     * Creates a response object by OAI verb and full, parameterized URL
     * @param $requestVerb The OAI verb
     * @param $requestUrl Full, parameterized URL
     */
    public function __construct($requestVerb, $requestUrl, $http_cache = FALSE) {
        $t0 = microtime(TRUE);
        OaiPmh::validateRequestVerb($requestVerb);
        $t1 = microtime(TRUE);
        $this->_statistics['resp_validate_verb'] = $t1 - $t0;
        
        //OaiPmh::validateUrl($requestUrl);
        $t2 = microtime(TRUE);
        $this->_statistics['resp_validate_url'] = $t2 - $t1;

        $this->_requestVerb = $requestVerb;
        $this->_requestUrl  = $requestUrl;
        $this->_http_cache  = $http_cache;
        $this->_setResponse();

        $t3 = microtime(TRUE);
        $this->_statistics['resp_http_response'] = $t3 - $t2;
    }
    
    public function getRespStatistics() {
      return $this->_statistics;
    }
    
    /**
     * Get the DOM object containing the parsed OAI-PMH. 
     * Returns an error object if the response is an error.
     * @return unknown_type
     */
    public function getDomObject() {
      if ($this->getHttpCode() != 200) {
        throw new Exception($this->_response);
      }
      
      if (OaiPmh_Dom::isErrorResponse($this->_response)) {
        $domClass = 'OaiPmh_Dom_Error';
      }
      else {
        $domClass = 'OaiPmh_Dom_' . $this->_requestVerb;
      }
      return new $domClass($this->_response);
    }
    
    protected function _setResponse() {
      $t0 = microtime(TRUE);
      if ($this->_http_cache !== FALSE) {
        $cache_file = $this->_http_cache . '/' . md5($this->_requestUrl);
      }
      if (isset($cache_file) && file_exists($cache_file)) {
        try {
          $this->_response = file_get_contents($cache_file);
        } catch (Exception $e) {
          xc_log_error('harvester', var_export($e, TRUE));
        }
        $this->_statistics['resp_curl_init'] = microtime(TRUE) - $t0;
        $this->_http_code = 200;
      }
      else {
        $ch = curl_init("");
        curl_setopt($ch, CURLOPT_HEADER, 0); //1
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
        curl_setopt($ch, CURLOPT_NOPROGRESS, 1);
        curl_setopt($ch, CURLOPT_USERAGENT, 'Omeka-XC OAI harvester');
        curl_setopt($ch, CURLOPT_ENCODING, "gzip,deflate");
        curl_setopt($ch, CURLOPT_CONNECTTIMEOUT, 10);
        if (substr($this->_requestUrl, 0, 5) == 'https') {
          if (preg_match('{^https://urresearch3.lib.rochester.edu}', $this->_requestUrl)) {
            curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, 0);
            // curl_setopt($ch, CURL_HTTP_VERSION, CURL_HTTP_VERSION_1_0);
            // curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, 2);
            $file = getcwd() . '/' . drupal_get_path('module', 'oaiharvester') . '/resources/urresearch3.lib.rochester.edu.crt';
            // xc_log_info('reponse', 'exists? ' . (int)file_exists($file) . ', ' . $file);
            //curl_setopt($ch, CURLOPT_CAINFO, $file);
          }
          else {
            // xc_log_info('reponse', '!urresearch3');
            curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, 0);
            //curl_setopt($ch, CURLOPT_CAINFO, 'cacert.pem');
          }
        }
        curl_setopt($ch, CURLOPT_URL, $this->_requestUrl);
        $this->_statistics['resp_curl_init'] = microtime(TRUE) - $t0;
        $this->_response = curl_exec($ch);
        // xc_log_info('response', CURLINFO_HEADER_OUT . ': '. var_export(curl_getinfo($ch, CURLINFO_HEADER_OUT), TRUE));
        $this->_http_code = curl_getinfo($ch, CURLINFO_HTTP_CODE);
        $download_size = curl_getinfo($ch, CURLINFO_SIZE_DOWNLOAD);
        if (isset($cache_file) && $this->_http_code == 200) {
          //file_save_data($this->_response, $cache_file, FILE_EXISTS_REPLACE);
          file_put_contents($cache_file, $this->_response);
          xc_log_info('harvester', 'cache file ' . $cache_file . ' created');
        }
        if (curl_errno($ch)) {
          $this->_response = curl_error($ch);
        }
        curl_close($ch);
        xc_log_info('reponse', 
          t('Received response. HTTP code: %code, size: %size.',
            array('%code' => $this->_http_code, '%size' => $download_size)));
      }
    }
    
    /**
     * Returns the raw HTTP response as string
     * @return unknown_type
     */
    public function getResponse() {
      return $this->_response;
    }
    
    public function getHttpCode() {
      return $this->_http_code;
    }
}
